﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class TimelineScript : MonoBehaviour
{
    public List<GameObject> timeline;
    public List<GameObject> corroborations;

    public GameObject corPanel;
    public GameObject submitButton;

    GameObject curSelected;

    public bool editActive;
    public Notebook notebook;
    public EvidenceHandler _handler;

    //checks if something has been added to the timeline for the tutorial
    public bool tutorialCheck;

    void Start()
    {
        // sets evidence handler
        _handler = FindObjectOfType<EvidenceHandler>();

        // sets all the timeline elements to the list
        foreach (Transform child in transform)
        {
            if(child.tag == "Timeline Element")
            {
                timeline.Add(child.gameObject);
                child.GetComponent<Button>().onClick.AddListener(() => TimelineClick(child.gameObject));
            }

        }

        // edit is set to not active
        editActive = false;

        tutorialCheck = false;
    }

    void Update()
    {   
        
    }

    // runs when edit button is pressed
    public void EditPressed()
    {
        // swaps whether its active or not
        editActive = !editActive;
        Debug.Log(editActive);

        if(editActive)
        {
            notebook.SelectedClear();
            // stops the player from corroborating
            notebook.canCorroborate = false;

            // clears corroboration list so multiples arent added
            corroborations.Clear();

            // resets corroboration list with all found corroborations
            foreach (Transform child in corPanel.transform)
            {
                corroborations.Add(child.gameObject);
            }

            // adds listened to all corroborations
            for(int i = 0; i < corroborations.Count; i++)
            {
                GameObject g = corroborations[i];
                Button b = g.GetComponent<Button>();
                b.onClick.AddListener(() => CorClick(g));
            }

            // sets timeline elements to be interactable
            for (int i = 0; i < timeline.Count; i++)
            {
                timeline[i].GetComponent<Button>().interactable = true;
            }


        } else
        {
            // resets everything when edit is pressed again
            notebook.canCorroborate = true;
            for (int i = 0; i < corroborations.Count; i++)
            {
                GameObject g = corroborations[i];
                Button b = g.GetComponent<Button>();
                b.onClick.RemoveListener(() => CorClick(g));
                Debug.Log("listener removed");
            }

            for (int i = 0; i < timeline.Count; i++)
            {
                timeline[i].GetComponent<Button>().interactable = false;
            }
            if(curSelected != null)
            {
                curSelected.GetComponent<Image>().color = Color.white;
                curSelected = null;
            }
        }
    }

    // runs when the evidence is clicked
    void CorClick(GameObject b)
    {
        if(editActive)
        {
            // checks if this evidence is already selected
            if (curSelected != b)
            {
                // if not, set it to selected and change the colour
                curSelected = b;
                b.GetComponent<Image>().color = Color.grey;
            }
            else
            {
                // deselects the element
                curSelected = null;
                b.GetComponent<Image>().color = Color.white;
            }
        }

    }

    // runs when a timeline element is clicked
    public void TimelineClick(GameObject g)
    {
        bool alreadyIn = false;
        // checks if anything is selected
        if (curSelected != null)
        {
            Text[] selectedInfo = curSelected.GetComponentsInChildren<Text>();
            for (int i = 0; i < timeline.Count; i++)
            {
                // checks if the selected item is already in the timeline or not
                Text check = timeline[i].GetComponentInChildren<Text>();

                if (check.text == selectedInfo[1].text)
                {
                    alreadyIn = true;
                }
            }
            if (!alreadyIn)
            {
                // if it's not already in the timeline, set that timeline element
                Text details = g.GetComponentInChildren<Text>();
                details.text = selectedInfo[1].text;

                EvidenceUI ev = g.AddComponent(typeof(EvidenceUI)) as EvidenceUI;
                ev.thisEvidenceID = curSelected.GetComponent<EvidenceUI>().thisEvidenceID;

                // deselects the element
                curSelected.GetComponent<Image>().color = Color.white;
                curSelected = null;

                // check position of timeline element
                int pos = -1;
                for(int i = 0; i < timeline.Count; i++)
                {
                    if(timeline[i] == g)
                    {
                        pos = i;
                    }
                }

                // sets it in the gameManager
                _handler.AddEvidenceToCaseSolution(ev, pos);
                // checks if the list is full
                if(_handler.CheckIfCaseSolutionIsFull())
                {
                    submitButton.GetComponent<Button>().interactable = true;
                } else
                {
                    submitButton.GetComponent<Button>().interactable = false;
                }
                tutorialCheck = true;
            }

        } else
        {
            // if nothing is selected, reset the timeline element
            Text details = g.GetComponentInChildren<Text>();
            details.text = "";

            // resets list element
            EvidenceUI nullEv = new EvidenceUI();
            nullEv.thisEvidenceID = "";
            
            int pos = -1; 
            for(int i =0; i < timeline.Count; i++)
            {
                if(timeline[i] == g)
                {
                    pos = i;
                }
            }

            _handler.AddEvidenceToCaseSolution(nullEv, pos);
        }
    }
}



