﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class WitnessScript : MonoBehaviour {

    public Personality personality;

    public Vector2 knownWeakStrongScale = new Vector2(-100.0f, 100.0f);
    public Vector2 knownDepravedHonorableScale = new Vector2(-100.0f, 100.0f);
    public Vector2 knownLazyAmbitiousScale = new Vector2(-100.0f, 100.0f);

    public BoxCollider witnessCollider;

    private GameObject knownStatsPanel;
    public GameObject prefabStatsPanel;
    public GameObject hoverParent;

    public bool beingQuestioned, interrogated;


    // Use this for initialization
    void Start ()
    {
        witnessCollider = this.GetComponent<BoxCollider>();

        // instantiating the known stats hover element.
        initialiseHover();

        beingQuestioned = false;

	}
	
	// Update is called once per frame
	void Update ()
    {
        updateStats();
	}

    public void makeStatement(float similarity)
    {
        float tickTime = 1.0f * (similarity / 100.0f);

        StartCoroutine(tickScaleKnowledge(tickTime));
    }

    public IEnumerator tickScaleKnowledge(float t)
    {
        float timer = t;

        while (timer > 0)
        {
            yield return new WaitForSeconds(Time.deltaTime);
            timer -= Time.deltaTime;
        }

        float whichScale = Random.Range(0.0f, 100.0f);
        
        if(whichScale < 33.3f)
        {
            narrowScale(ref knownWeakStrongScale, personality.WeakStrongScale);
        }
        else if (whichScale >= 33.3f && whichScale < 66.6f)
        {
            narrowScale(ref knownDepravedHonorableScale, personality.DepravedHonorableScale);
        }
        else if (whichScale >= 66.6f)
        {
            narrowScale(ref knownLazyAmbitiousScale, personality.LazyAmbitiousScale);
        }

        if(beingQuestioned)
        {
            StartCoroutine(tickScaleKnowledge(t));
        }
    }

    void narrowScale(ref Vector2 scale, float personalityItem)
    {
        float r = Random.Range(scale.x, scale.y);

        if(r < personalityItem && scale.x != personalityItem)
        {
            scale.x += 5;
        }
        else if(r > personalityItem && scale.y != personalityItem)
        {
            scale.y -= 5;
        }
    }

    void initialiseHover()
    {
        // instantiate stats panel
        knownStatsPanel = Instantiate(prefabStatsPanel) as GameObject;

        Text panelName = knownStatsPanel.GetComponentInChildren<Text>();
        panelName.text = this.name;

        knownStatsPanel.transform.SetParent(hoverParent.transform, false);

        updateStats();
        KnownStatsScript statsScript = this.GetComponent<KnownStatsScript>();
        statsScript.setPanel(knownStatsPanel);
    }

    void updateStats()
    {
        Slider[] stats = knownStatsPanel.GetComponentsInChildren<Slider>();

        stats[0].value = -100 - knownWeakStrongScale[0] + stats[0].maxValue;
        stats[1].value = knownWeakStrongScale[1];
        stats[2].value = -100 - knownDepravedHonorableScale[0] + stats[2].maxValue;
        stats[3].value = knownDepravedHonorableScale[1];
        stats[4].value = -100 - knownLazyAmbitiousScale[0] + stats[4].maxValue;
        stats[5].value = knownLazyAmbitiousScale[1];
        
        if(knownWeakStrongScale[0] > stats[1].minValue)
        {
            float change = knownWeakStrongScale[0];

            float width = stats[0].gameObject.GetComponent<RectTransform>().sizeDelta.x;
            float widthChange = (width / (100 + stats[1].minValue)) * (change - stats[1].minValue);

            Vector2 sizeD0 = stats[0].gameObject.GetComponent<RectTransform>().sizeDelta;
            stats[0].gameObject.GetComponent<RectTransform>().sizeDelta = new Vector2(sizeD0.x + widthChange, sizeD0.y);
            Vector3 pos0 = stats[0].gameObject.GetComponent<RectTransform>().transform.position;
            stats[0].gameObject.GetComponent<RectTransform>().transform.position = new Vector3(pos0.x + (widthChange/2), pos0.y, pos0.z);

            Vector2 sizeD1 = stats[1].gameObject.GetComponent<RectTransform>().sizeDelta;
            stats[1].gameObject.GetComponent<RectTransform>().sizeDelta = new Vector2(sizeD1.x - widthChange, sizeD1.y);
            Vector3 pos1 = stats[1].gameObject.GetComponent<RectTransform>().transform.position;
            stats[1].gameObject.GetComponent<RectTransform>().transform.position = new Vector3(pos1.x + (widthChange / 2), pos1.y, pos1.z);

            stats[0].maxValue = change;
            stats[1].minValue = change;
        }

        if(knownWeakStrongScale[1] < stats[0].maxValue)
        {
            float change = knownWeakStrongScale[1];

            float width = stats[1].gameObject.GetComponent<RectTransform>().sizeDelta.x;
            float widthChange = (width / (100 - stats[0].maxValue)) * (change - stats[0].maxValue);

            Vector2 sizeD0 = stats[0].gameObject.GetComponent<RectTransform>().sizeDelta;
            stats[0].gameObject.GetComponent<RectTransform>().sizeDelta = new Vector2(sizeD0.x + widthChange, sizeD0.y);
            Vector3 pos0 = stats[0].gameObject.GetComponent<RectTransform>().transform.position;
            stats[0].gameObject.GetComponent<RectTransform>().transform.position = new Vector3(pos0.x + (widthChange / 2), pos0.y, pos0.z);

            Vector2 sizeD1 = stats[1].gameObject.GetComponent<RectTransform>().sizeDelta;
            stats[1].gameObject.GetComponent<RectTransform>().sizeDelta = new Vector2(sizeD1.x - widthChange, sizeD1.y);
            Vector3 pos1 = stats[1].gameObject.GetComponent<RectTransform>().transform.position;
            stats[1].gameObject.GetComponent<RectTransform>().transform.position = new Vector3(pos1.x + (widthChange / 2), pos1.y, pos1.z);

            stats[0].maxValue = change;
            stats[1].minValue = change;
        }

        if (knownDepravedHonorableScale[0] > stats[3].minValue)
        {
            float change = knownWeakStrongScale[0];

            float width = stats[2].gameObject.GetComponent<RectTransform>().sizeDelta.x;
            float widthChange = (width / (100 + stats[3].minValue)) * (change - stats[3].minValue);

            Vector2 sizeD2 = stats[2].gameObject.GetComponent<RectTransform>().sizeDelta;
            stats[2].gameObject.GetComponent<RectTransform>().sizeDelta = new Vector2(sizeD2.x + widthChange, sizeD2.y);
            Vector3 pos2 = stats[2].gameObject.GetComponent<RectTransform>().transform.position;
            stats[2].gameObject.GetComponent<RectTransform>().transform.position = new Vector3(pos2.x + (widthChange / 2), pos2.y, pos2.z);

            Vector2 sizeD3 = stats[3].gameObject.GetComponent<RectTransform>().sizeDelta;
            stats[3].gameObject.GetComponent<RectTransform>().sizeDelta = new Vector2(sizeD3.x - widthChange, sizeD3.y);
            Vector3 pos3 = stats[3].gameObject.GetComponent<RectTransform>().transform.position;
            stats[3].gameObject.GetComponent<RectTransform>().transform.position = new Vector3(pos3.x + (widthChange / 2), pos3.y, pos3.z);

            stats[2].maxValue = change;
            stats[3].minValue = change;
        }

        if (knownDepravedHonorableScale[1] < stats[2].maxValue)
        {
            float change = knownDepravedHonorableScale[1];

            float width = stats[3].gameObject.GetComponent<RectTransform>().sizeDelta.x;
            float widthChange = (width / (100 - stats[2].maxValue)) * (change - stats[2].maxValue);

            Vector2 sizeD2 = stats[2].gameObject.GetComponent<RectTransform>().sizeDelta;
            stats[2].gameObject.GetComponent<RectTransform>().sizeDelta = new Vector2(sizeD2.x + widthChange, sizeD2.y);
            Vector3 pos2 = stats[2].gameObject.GetComponent<RectTransform>().transform.position;
            stats[2].gameObject.GetComponent<RectTransform>().transform.position = new Vector3(pos2.x + (widthChange / 2), pos2.y, pos2.z);

            Vector2 sizeD3 = stats[3].gameObject.GetComponent<RectTransform>().sizeDelta;
            stats[3].gameObject.GetComponent<RectTransform>().sizeDelta = new Vector2(sizeD3.x - widthChange, sizeD3.y);
            Vector3 pos3 = stats[3].gameObject.GetComponent<RectTransform>().transform.position;
            stats[3].gameObject.GetComponent<RectTransform>().transform.position = new Vector3(pos3.x + (widthChange / 2), pos3.y, pos3.z);

            stats[2].maxValue = change;
            stats[3].minValue = change;
        }

        if (knownLazyAmbitiousScale[0] > stats[5].minValue)
        {
            float change = knownWeakStrongScale[0];

            float width = stats[4].gameObject.GetComponent<RectTransform>().sizeDelta.x;
            float widthChange = (width / (100 + stats[5].minValue)) * (change - stats[5].minValue);

            Debug.Log(widthChange);

            Vector2 sizeD4 = stats[4].gameObject.GetComponent<RectTransform>().sizeDelta;
            stats[4].gameObject.GetComponent<RectTransform>().sizeDelta = new Vector2(sizeD4.x + widthChange, sizeD4.y);
            Vector3 pos4 = stats[4].gameObject.GetComponent<RectTransform>().transform.position;
            stats[4].gameObject.GetComponent<RectTransform>().transform.position = new Vector3(pos4.x + (widthChange / 2), pos4.y, pos4.z);

            Vector2 sizeD5 = stats[5].gameObject.GetComponent<RectTransform>().sizeDelta;
            stats[5].gameObject.GetComponent<RectTransform>().sizeDelta = new Vector2(sizeD5.x - widthChange, sizeD5.y);
            Vector3 pos5 = stats[5].gameObject.GetComponent<RectTransform>().transform.position;
            stats[5].gameObject.GetComponent<RectTransform>().transform.position = new Vector3(pos5.x + (widthChange / 2), pos5.y, pos5.z);

            stats[4].maxValue = change;
            stats[5].minValue = change;
        }

        if (knownLazyAmbitiousScale[1] < stats[4].maxValue)
        {
            float change = knownLazyAmbitiousScale[1];

            float width = stats[5].gameObject.GetComponent<RectTransform>().sizeDelta.x;
            float widthChange = (width / (100 - stats[4].maxValue)) * (change - stats[4].maxValue);

            Vector2 sizeD4 = stats[4].gameObject.GetComponent<RectTransform>().sizeDelta;
            stats[4].gameObject.GetComponent<RectTransform>().sizeDelta = new Vector2(sizeD4.x + widthChange, sizeD4.y);
            Vector3 pos4 = stats[4].gameObject.GetComponent<RectTransform>().transform.position;
            stats[4].gameObject.GetComponent<RectTransform>().transform.position = new Vector3(pos4.x + (widthChange / 2), pos4.y, pos4.z);

            Vector2 sizeD5 = stats[5].gameObject.GetComponent<RectTransform>().sizeDelta;
            stats[5].gameObject.GetComponent<RectTransform>().sizeDelta = new Vector2(sizeD5.x - widthChange, sizeD5.y);
            Vector3 pos5 = stats[5].gameObject.GetComponent<RectTransform>().transform.position;
            stats[5].gameObject.GetComponent<RectTransform>().transform.position = new Vector3(pos5.x + (widthChange / 2), pos5.y, pos5.z);

            stats[4].maxValue = change;
            stats[5].minValue = change;
        }
    }
}
